@@ -36,6 +36,7 @@
 "*mitSaverExtension:	False",
 "*xidleExtension:	True",
 "*procInterrupts:	True",
+"GetViewPortIsFullOfLies: False",
 "*demoCommand: xscreensaver-demo",
 "*prefsCommand: xscreensaver-demo -prefs",
 "*helpURL: http://www.jwz.org/xscreensaver/man.html",
@@ -48,7 +49,7 @@
 	   \"Qix (transparent)\" 	qix -root -count 4 -solid -transparent	    \\n\
 		\"Qix (linear)\" 	qix -root -count 5 -solid -transparent	      \
 				  -linear -segments 250 -size 100	    \\n\
-- mono: 	   \"Qix (xor)\" 	qix -root -linear -count 5 -size 200	      \
+-		   \"Qix (xor)\" 	qix -root -linear -count 5 -size 200	      \
 				  -spread 30 -segments 75 -solid -xor	    \\n\
 									      \
 	  \"Attraction (balls)\" 	attraction -root -mode balls		    \\n\
@@ -79,7 +80,7 @@
 	      \"Ripples (stir)\"	ripples -root -oily -light 2 -stir	    \\n\
 	   \"Ripples (desktop)\"	ripples -root -water -light 6		    \\n\
 				hypercube -root				    \\n\
-				hyperball -root				    \\n\
+-				hyperball -root				    \\n\
 				halo -root				    \\n\
 				maze -root				    \\n\
 				noseguy -root				    \\n\
@@ -113,6 +114,7 @@
 				starfish -root				    \\n\
 	     \"Starfish (blob)\" 	starfish -root -blob			    \\n\
 				munch -root				    \\n\
+				mismunch -root				    \\n\
 				fadeplot -root				    \\n\
 				coral -root -delay 0			    \\n\
 				mountain -root				    \\n\
@@ -160,6 +162,7 @@
 				blaster -root				    \\n\
 				bumps -root				    \\n\
 				xteevee -root				    \\n\
+				xanalogtv -root				    \\n\
 				xspirograph -root			    \\n\
 				nerverot -root				    \\n\
 -	    \"NerveRot (dense)\"	nerverot -root -count 1000		    \\n\
@@ -187,7 +190,20 @@
  	                	fluidballs -root			    \\n\
  	                	anemone -root				    \\n\
  	                	halftone -root				    \\n\
-  color: 			bubbles -root				    \\n\
+ 	                	metaballs -root				    \\n\
+ 	                	eruption -root				    \\n\
+ 	                	popsquares -root			    \\n\
+ 	                	barcode -root				    \\n\
+ 	                	piecewise -root				    \\n\
+ 	                	cloudlife -root				    \\n\
+		   \"FontGlide\"	fontglide -root -page			    \\n\
+	\"FontGlide (scroller)\"	fontglide -root -scroll			    \\n\
+				apple2 -root				    \\n\
+                                bubbles -root				    \\n\
+				pong -root				    \\n\
+				wormhole -root				    \\n\
+				pacman -root				    \\n\
+				fuzzyflakes -root			    \\n\
 - default-n:			webcollage -root			    \\n\
 - default-n:  \"WebCollage (whacked)\"					      \
 				webcollage -root -filter		      \
@@ -208,11 +224,12 @@
 	   GL:			lament -root				    \\n\
 	   GL:			bubble3d -root				    \\n\
 	   GL:			glplanet -root				    \\n\
+	   GL:			flurry -root -preset random		    \\n\
 	   GL:			pulsar -root				    \\n\
 -	   GL:	   \"Pulsar (textures)\"					      \
 				  pulsar -root -texture -mipmap		      \
 				  -texture_quality -light -fog		    \\n\
-	   GL:			extrusion -root				    \\n\
+-	   GL:			extrusion -root				    \\n\
 	   GL:			sierpinski3d -root			    \\n\
 	   GL:			menger -root				    \\n\
 	   GL:	 \"GFlux\"	gflux -root				    \\n\
@@ -229,7 +246,8 @@
 	   GL:			flipscreen3d -root			    \\n\
 	   GL:			glsnake -root				    \\n\
 	   GL:			boxed -root				    \\n\
-	   GL:			glforestfire -root			    \\n\
+	   GL:	\"GLForestFire\"		glforestfire -root		    \\n\
+-	   GL:	\"GLForestFire (rain)\"	glforestfire -root -count 0	    \\n\
 	   GL:			sballs -root				    \\n\
 	   GL:			cubenetic -root				    \\n\
 	   GL:			spheremonics -root			    \\n\
@@ -237,13 +255,35 @@
 	   GL:			queens -root				    \\n\
 	   GL:			endgame -root				    \\n\
 -	   GL:			glblur -root				    \\n\
+	   GL:			atunnel -root				    \\n\
+	   GL:			flyingtoasters -root			    \\n\
+	   GL:			bouncingcow -root			    \\n\
+	   GL:			jigglypuff -root -random		    \\n\
+	   GL:			klein -root -random			    \\n\
+	   GL:	\"HyperTorus (striped)\" hypertorus -root			    \\n\
+	   GL:	\"HyperTorus (solid)\"   hypertorus -root -solid -transparent \\n\
+	   GL:			glmatrix -root				    \\n\
+	   GL:			cubestorm -root				    \\n\
+	   GL:			glknots -root				    \\n\
+	   GL:			blocktube -root				    \\n\
+	   GL:			flipflop -root				    \\n\
+	   GL:			antspotlight -root			    \\n\
+-	   GL:			glslideshow -root			    \\n\
+	   GL:			polytopes -root				    \\n\
+	   GL:			gleidescope -root			    \\n\
+- 	   GL:			mirrorblob -root			    \\n\
+	   GL:	    \"MirrorBlob (color only)\"				      \
+                                mirrorblob -root -colour -no-texture	    \\n\
+	   GL:			blinkbox -root				    \\n\
+	   GL:			noof -root				    \\n\
+	   GL:			polyhedra -root				    \\n\
 									      \
 -				xdaliclock -root -builtin3 -cycle	    \\n\
 - default-n:			xearth -nofork -nostars -ncolors 50	      \
 				  -night 3 -wait 0 -timewarp 400.0 -pos	      \
 				  sunrel/38/-30				    \\n\
--				xplanetbg -xscreensaver -moonside             \
-                                  -markerfile earth -wait 1 -timewarp 400   \\n\
+-				xplanet -vroot -wait 1 -timewarp 90000        \
+                                  -label -origin moon			    \\n\
 -				xmountains -b -M -Z 0 -r 1		    \\n\
 -	\"XMountains (top)\"	xmountains -b -M -Z 0 -r 1 -m		    \\n\
 -                               xaos -root -autopilot -nogui -delay 10000     \
@@ -254,7 +294,10 @@
 -				goban -root                                 \\n\
 -				electricsheep                               \\n\
 -				cosmos -root                                \\n\
--	   GL:                  sphereEversion --root                       \\n",
+-	   GL:                  sphereEversion --root                       \\n\
+-	   GL:                  fireflies -root                             \\n\
+-	   GL:                  antinspect -root                            \\n\
+-	   GL:                  providence -root                            \\n",
 "XScreenSaver.pointerPollTime:		0:00:05",
 "XScreenSaver.initialDelay:		0:00:00",
 "XScreenSaver.windowCreationTimeout:	0:00:30",
@@ -286,7 +329,7 @@
 "*passwd.passwdFont:		*-courier-medium-r-*-*-*-140-*-*-*-iso8859-1",
 "*passwd.thermometer.width:	8",
 "*splash.heading.label:		XScreenSaver %s",
-"*splash.body.label:		Copyright  1991-2002 by",
+"*splash.body.label:		Copyright  1991-2004 by",
 "*splash.body2.label:		Jamie Zawinski <jwz@jwz.org>",
 "*splash.demo.label:		Settings",
 "*splash.help.label:		Help",
@@ -411,13 +454,13 @@
 "*hacks.shadebobs.name:      ShadeBobs",
 "*hacks.ccurve.name:         C Curve",
 "*hacks.xteevee.name:        XTeeVee",
+"*hacks.xanalogtv.name:      XAnalogTV",
 "*hacks.xspirograph.name:    XSpiroGraph",
 "*hacks.nerverot.name:       NerveRot",
 "*hacks.webcollage.name:     WebCollage",
 "*hacks.vidwhacker.name:     VidWhacker",
 "*hacks.morph3d.name:        Morph3D",
 "*hacks.bubble3d.name:       Bubble3D",
-"*hacks.glplanet.name:       GLPlanet",
 "*hacks.sierpinski3d.name:   Sierpinski3D",
 "*hacks.gflux.name:          GFlux",
 "*hacks.xrayswarm.name:      XRaySwarm",
@@ -425,19 +468,29 @@
 "*hacks.rotzoomer.name:      RotZoomer",
 "*hacks.stonerview.name:     StonerView",
 "*hacks.starwars.name:       StarWars",
-"*hacks.glblur.name:         GLBlur",
-"*hacks.gltext.name:         GLText",
 "*hacks.dangerball.name:     DangerBall",
 "*hacks.whirlygig.name:      WhirlyGig",
 "*hacks.speedmine.name:      SpeedMine",
-"*hacks.glsnake.name:        GLSnake",
 "*hacks.glforestfire.name:   GLForestFire",
 "*hacks.sballs.name:         SBalls",
 "*hacks.xdaliclock.name:     XDaliClock",
 "*hacks.xplanetbg.name:      XPlanet",
+"*hacks.xplanet.name:        XPlanet",
 "*hacks.xaos.name:           XaoS",
 "*hacks.xfishtank.name:      XFishTank",
 "*hacks.electricsheep.name:  ElectricSheep",
 "*hacks.sphereEversion.name: SphereEversion",
 "*hacks.fluidballs.name:     FluidBalls",
+"*hacks.flyingtoasters.name: FlyingToasters",
+"*hacks.bouncingcow.name:    BouncingCow",
+"*hacks.jigglypuff.name:     JigglyPuff",
+"*hacks.hypertorus.name:     HyperTorus",
+"*hacks.cubestorm.name:      CubeStorm",
+"*hacks.blocktube.name:      BlockTube",
+"*hacks.flipflop.name:       FlipFlop",
+"*hacks.antspotlight.name:   AntSpotlight",
+"*hacks.fontglide.name:      FontGlide",
+"*hacks.mirrorblob.name:     MirrorBlob",
+"*hacks.blinkbox.name:       BlinkBox",
+"*hacks.fuzzyflakes.name:    FuzzyFlakes",
 "*hacks.documentation.isInstalled: True",
