@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1993-2002 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1993-2004 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -20,6 +20,10 @@
 #include <string.h>
 #include <stdio.h>
 
+#ifdef HAVE_SIGACTION
+# include <signal.h>    /* for sigset_t */
+#endif
+
 #include "prefs.h"
 
 extern char *progname;
@@ -62,6 +66,7 @@ struct saver_info {
      server extension info
      ======================================================================= */
 
+  Bool xinerama_p;		   /* Whether Xinerama is in use.            */
   Bool using_xidle_extension;	   /* which extension is being used.         */
   Bool using_mit_saver_extension;  /* Note that `p->use_*' is the *request*, */
   Bool using_sgi_saver_extension;  /* and `si->using_*' is the *reality*.    */
@@ -75,6 +80,10 @@ struct saver_info {
   int sgi_saver_ext_event_number;
   int sgi_saver_ext_error_number;
 # endif
+# ifdef HAVE_RANDR
+  int randr_event_number;
+  int randr_error_number;
+# endif
 
 
   /* =======================================================================
@@ -184,8 +193,19 @@ struct saver_info {
 struct saver_screen_info {
   saver_info *global;
 
-  int number;
-  Screen *screen;
+  int number;			/* The internal ordinal of this screen,
+                                   counting Xinerama rectangles as separate
+                                   screens. */
+  int real_screen_number;	/* The number of the underlying X screen on
+                                   which this rectangle lies. */
+  Screen *screen;		/* The X screen in question. */
+
+  int x, y, width, height;	/* The size and position of this rectangle
+                                   on its underlying X screen. */
+
+  Bool real_screen_p;		/* This will be true of exactly one ssi per
+                                   X screen. */
+
   Widget toplevel_shell;
 
   /* =======================================================================
@@ -283,6 +303,9 @@ extern Bool query_sgi_saver_extension (saver_info *);
 #ifdef HAVE_XIDLE_EXTENSION
 extern Bool query_xidle_extension (saver_info *);
 #endif
+#ifdef HAVE_RANDR
+extern Bool query_randr_extension (saver_info *);
+#endif
 #ifdef HAVE_PROC_INTERRUPTS
 extern Bool query_proc_interrupts_available (saver_info *, const char **why);
 #endif
@@ -302,6 +325,7 @@ extern void raise_window (saver_info *si,
 			    Bool dont_clear);
 extern Bool blank_screen (saver_info *si);
 extern void unblank_screen (saver_info *si);
+extern void resize_screensaver_window (saver_info *si);
 
 extern void get_screen_viewport (saver_screen_info *ssi,
                                  int *x_ret, int *y_ret,
@@ -374,7 +398,11 @@ extern void maybe_reload_init_file (saver_info *);
    ======================================================================= */
 
 extern void handle_signals (saver_info *si);
-extern void block_sigchld (void);
+#ifdef HAVE_SIGACTION
+ extern sigset_t block_sigchld (void);
+#else  /* !HAVE_SIGACTION */
+ extern int block_sigchld (void);
+#endif /* !HAVE_SIGACTION */
 extern void unblock_sigchld (void);
 extern void hack_environment (saver_info *si);
 extern void hack_subproc_environment (saver_screen_info *ssi);
