@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1997, 1998 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1997, 1998, 2002 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -14,6 +14,8 @@
 
 #include "screenhack.h"
 #include <stdio.h>
+#include <time.h>
+#include <sys/time.h>
 
 static XColor colors[255];
 static int ncolors = 0;
