@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* drift --- drifting recursive fractal cosmic flames */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)drift.c	5.00 2000/11/01 xlockmore";
-
 #endif
 
 /*-
@@ -69,8 +68,8 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & grow, (char *) "grow", (char *) "Grow", (char *) DEF_GROW, t_Bool},
-	{(caddr_t *) & liss, (char *) "liss", (char *) "Liss", (char *) DEF_LISS, t_Bool}
+	{&grow, "grow", "Grow", DEF_GROW, t_Bool},
+	{&liss, "liss", "Liss", DEF_LISS, t_Bool}
 };
 static OptionStruct desc[] =
 {
