@@ -1,6 +1,6 @@
 /* sballs --- balls spinning like crazy in GL */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
 #endif
 
@@ -74,16 +74,25 @@ static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
 
 #if defined( USE_XPM ) || defined( USE_XPMINC ) || defined( HAVE_XPM )
 /* USE_XPM & USE_XPMINC in xlock mode ; HAVE_XPM in xscreensaver mode */
-#include "xpm-ximage.h"
-#define I_HAVE_XPM
+# include "xpm-ximage.h"
+# define I_HAVE_XPM
 
-#ifdef STANDALONE
-#include "../images/sball.xpm"
-#include "../images/sball-bg.xpm"
-#else /* !STANDALONE */
-#include "pixmaps/sball.xpm"
-#include "pixmaps/sball-bg.xpm"
-#endif /* !STANDALONE */
+# ifdef STANDALONE
+
+#  ifdef __GNUC__
+   __extension__ /* don't warn about "string length is greater than the length
+                    ISO C89 compilers are required to support" when including
+                    the following XPM file... */
+#  endif
+#  include "../images/sball.xpm"
+#  ifdef __GNUC__
+   __extension__
+#  endif
+#  include "../images/sball-bg.xpm"
+# else /* !STANDALONE */
+#  include "pixmaps/sball.xpm"
+#  include "pixmaps/sball-bg.xpm"
+# endif /* !STANDALONE */
 #endif /* HAVE_XPM */
 
 /* Manage option vars */
@@ -106,9 +115,9 @@ static XrmOptionDescRec opts[] = {
 };
 
 static argtype vars[] = {
-    {(caddr_t *) & do_texture, (char *) "texture", (char *) "Texture", (char *) DEF_TEXTURE, t_Bool},
-    {(caddr_t *) & do_trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool},
-    {(caddr_t *) & object, (char *) "object", (char *) "Object", (char *) DEF_OBJECT, t_Int},
+    {&do_texture,    "texture",    "Texture",    DEF_TEXTURE,    t_Bool},
+    {&do_trackmouse, "trackmouse", "TrackMouse", DEF_TRACKMOUSE, t_Bool},
+    {&object,        "object",     "Object",     DEF_OBJECT,     t_Int},
 
 };
 
@@ -441,7 +450,10 @@ static void inittextures(ModeInfo * mi)
         clear_gl_error();
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
 		     sb->btexture->width, sb->btexture->height, 0,
-		     GL_RGBA, GL_UNSIGNED_BYTE, sb->btexture->data);
+		     GL_RGBA,
+                     /* GL_UNSIGNED_BYTE, */
+                     GL_UNSIGNED_INT_8_8_8_8_REV,
+                     sb->btexture->data);
         check_gl_error("texture");
 
 	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
@@ -472,7 +484,10 @@ static void inittextures(ModeInfo * mi)
         clear_gl_error();
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
 		     sb->ftexture->width, sb->ftexture->height, 0,
-		     GL_RGBA, GL_UNSIGNED_BYTE, sb->ftexture->data);
+		     GL_RGBA,
+                     /* GL_UNSIGNED_BYTE, */
+                     GL_UNSIGNED_INT_8_8_8_8_REV,
+                     sb->ftexture->data);
         check_gl_error("texture");
 
 	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
@@ -707,7 +722,7 @@ static void Init(ModeInfo * mi)
 		       MI_NAME(mi),
 		       polygons[object].shortname,
 		       spheres,
-		       MI_CYCLES(mi),
+		       (int) MI_CYCLES(mi),
 		       do_texture ? "on" : "off"
 			);
     }
@@ -855,7 +870,7 @@ void change_sballs(ModeInfo * mi)
 		       MI_NAME(mi),
 		       polygons[object].shortname,
 		       spheres,
-		       MI_CYCLES(mi),
+		       (int) MI_CYCLES(mi),
 		       do_texture ? "on" : "off"
 			);
     }
