@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4 -*-
  * julia --- continuously varying Julia set.
  */
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)julia.c	4.03 97/04/10 xlockmore";
 #endif
 
@@ -20,8 +20,8 @@ static const char sccsid[] = "@(#)julia.c	4.03 97/04/10 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 28-May-97: jwz@netscape.com: added interactive frobbing with the mouse.
- * 10-May-97: jwz@netscape.com: turned into a standalone program.
+ * 28-May-97: jwz@jwz.org: added interactive frobbing with the mouse.
+ * 10-May-97: jwz@jwz.org: turned into a standalone program.
  * 02-Dec-95: snagged boilerplate from hop.c
  *           used ifs {w0 = sqrt(x-c), w1 = -sqrt(x-c)} with random iteration 
  *           to plot the julia set, and sinusoidially varied parameter for set 
@@ -63,7 +63,7 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & track_p, "mouse", "Mouse", DEF_MOUSE, t_Bool},
+	{&track_p, "mouse", "Mouse", DEF_MOUSE, t_Bool},
 };
 static OptionStruct desc[] =
 {
